/*
 * WItalk2 - italk client for Win32
 * Copyright (C) 1996-1997, TAGA Nayuta <nayuta@is.s.u-tokyo.ac.jp>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

///////////////////////////////////////////////////////////////////////////////
// DialogSelectSound.cxx


#include	<winsock2.h>
#include "Dialog.h"
#include "lgDialog.h"
#include "Sound.h"


class DialogSelectSound : public lgDialog
{
  Sound editing_sound;
  Sound *sound;
  BOOL enable_update;
  
  void set_all(void)
  {
    int s = editing_sound.GetSound();
    BOOL does_enable = (s != -3);
    EnableWindow   (Hwnd(rRadioBeepFile),        does_enable);
    EnableWindow   (Hwnd(rRadioBeep),            does_enable);
    EnableWindow   (Hwnd(rRadioBeepAsterisk),    does_enable);
    EnableWindow   (Hwnd(rRadioBeepHand),        does_enable);
    EnableWindow   (Hwnd(rRadioBeepQuestion),    does_enable);
    EnableWindow   (Hwnd(rRadioBeepOk),          does_enable);
    EnableWindow   (Hwnd(rRadioBeepExclamation), does_enable);
    EnableWindow   (Hwnd(rButtonBeepTest),       does_enable);
    EnableWindow   (Hwnd(rButtonBeepBrowse),     does_enable && s == -2);
    EnableWindow   (Hwnd(rEditBeepFile),         does_enable && s == -2);
    Button_SetCheck(Hwnd(rCheckDoesPlaySound),   s != -3);
    Button_SetCheck(Hwnd(rRadioBeepFile),        s == -2);
    Button_SetCheck(Hwnd(rRadioBeep),            s == -1);
    Button_SetCheck(Hwnd(rRadioBeepAsterisk),    s == MB_ICONASTERISK);
    Button_SetCheck(Hwnd(rRadioBeepHand),        s == MB_ICONHAND);
    Button_SetCheck(Hwnd(rRadioBeepQuestion),    s == MB_ICONQUESTION);
    Button_SetCheck(Hwnd(rRadioBeepOk),          s == MB_OK);
    Button_SetCheck(Hwnd(rRadioBeepExclamation), s == MB_ICONEXCLAMATION);
  }
  
public:
  DialogSelectSound(HWND _hwnd)
    : lgDialog(_hwnd),
      enable_update(FALSE)
  {
  }
  
  // WM_INITDIALOG
  BOOL wmInitDialog(HWND focus, LPARAM lParam)
  {
    HANDLE hicon = LoadIcon(lgWindows::hInst(), MAKEINTRESOURCE(rIconWItalk2));
    SetClassLong(Hwnd(), GCL_HICON, (LONG)hicon);
    
    sound = (Sound *)lParam;
    editing_sound = *sound;
    
    if (editing_sound.GetFilename())
      SetWindowText(rEditBeepFile, editing_sound.GetFilename());

    enable_update = TRUE;
    
    set_all();
    
    return TRUE;
  }

  // WM_COMMAND
  BOOL wmCommand(int notify_code, int id, HWND /* hwnd_control */)
  {
    switch (id)
    {
#define c(x, y) case r##x: return wmCommand_RadioBeep(y)
      c(RadioBeep,            -1);
      c(RadioBeepAsterisk,    MB_ICONASTERISK);
      c(RadioBeepExclamation, MB_ICONEXCLAMATION);
      c(RadioBeepHand,        MB_ICONHAND);
      c(RadioBeepQuestion,    MB_ICONQUESTION);
      c(RadioBeepOk,          MB_OK);
      c(RadioBeepFile,        -2);
#undef c
#define c(x) case r##x: return wmCommand_##x(notify_code)
      c(CheckDoesPlaySound);
      c(ButtonBeepTest);
      c(ButtonBeepBrowse);
      c(EditBeepFile);
#undef c
      
      case IDOK:     return wmCommand_IDOK();
      case IDCANCEL: return wmClose();
    }
    return TRUE;
  }

  // ݒ
  BOOL wmCommand_RadioBeep(int sound)
  {
    if (enable_update)
    {
      char buf[BUFSIZE];
      GetWindowText(rEditBeepFile, buf, sizeof(buf));
      editing_sound.Set(sound, buf);
      set_all();
    }
    return TRUE;
  }
  
  // 炷ǂ
  BOOL wmCommand_CheckDoesPlaySound(int command)
  {
    if (Button_GetCheck(Hwnd(rCheckDoesPlaySound)))
      editing_sound.Set(MB_OK);
    else
      editing_sound.Set(-3);
    set_all();
    return TRUE;
  }
  
  // t@C
  BOOL wmCommand_EditBeepFile(int command)
  {
    if (command == EN_UPDATE)
      wmCommand_RadioBeep(-2);
    return TRUE;
  }
  
  // eXg
  BOOL wmCommand_ButtonBeepTest(int command)
  {
    if (command == BN_CLICKED)
      editing_sound.Play();
    return TRUE;
  }
  
  // Q
  BOOL wmCommand_ButtonBeepBrowse(int command)
  {
    if (command == BN_CLICKED)
    {
      char filename[BUFSIZE] = "";
      char filter[BUFSIZE];
      lgString::ReplaceChar(strcpy(filter, LoadString(rStringFilterWAV)));
      OPENFILENAME of;
      
      ZeroMemory(&of, sizeof(OPENFILENAME));
      of.lStructSize     = sizeof(OPENFILENAME);
      of.hwndOwner       = Hwnd();
      of.hInstance       = lgWindows::hInst();
      of.lpstrFilter     = filter;
      of.nFilterIndex    = 0;
      of.lpstrFile       = filename;
      of.nMaxFile        = sizeof(filename);
      of.lpstrInitialDir = NULL;
      of.lpstrTitle      = LoadString(rStringOpenWave);
      of.Flags           = OFN_FILEMUSTEXIST | OFN_HIDEREADONLY |
	OFN_NOTESTFILECREATE | OFN_PATHMUSTEXIST | OFN_EXPLORER;
      
      if (GetOpenFileName(&of))
	SetWindowText(rEditBeepFile, filename);
    }
    return TRUE;
  }
  
  // Z[u
  BOOL wmCommand_IDOK(void)
  {
    wmCommand_RadioBeep(editing_sound.GetSound());
    *sound = editing_sound;
    EndDialog(Hwnd(), (LPARAM)sound);
    return TRUE;
  }
  
  // 
  BOOL wmClose(void)
  {
    EndDialog(Hwnd(), NULL);
    return TRUE;
  }
};


BOOL CALLBACK DialogSelectSoundDlgProc(HWND hwnd, UINT message,
				   WPARAM wParam, LPARAM lParam)
{
#define lgWindowClass DialogSelectSound
  if (!lgwc)
    switch (message)
    {
      lgwmInitDialog;
    }
  else
    switch (message)
    {
      lgwmCommand;
      lgwmClose;
    }
  return lgwc->Default(message, wParam, lParam);
}
