// Italk Protocol 1.0 and biff-port support modifications.
// Copyright (C) 1998, SAKAMOTO Takahiro <takas@is.s.u-tokyo.ac.jp>

/*
 * WItalk2 - italk client for Win32
 * Copyright (C) 1996-1997, TAGA Nayuta <nayuta@is.s.u-tokyo.ac.jp>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

///////////////////////////////////////////////////////////////////////////////
// ItalkConnection.cxx


#include	<winsock2.h>
#include	<new>
using namespace std;

#include	"iWItalk2.h"
#include	"iso2022.h"
#include	"witalk2.h"
#include	"Trace.h"

#define		VA_SIZE		4096

extern	WItalk2* witalk2_ptr;

BOOL ItalkConnection::check_read_buffer(void)
{
	if (read_buffer_commit <= read_pointer + 1) {
		if (read_buffer_reserve < read_buffer_commit + VA_SIZE) return FALSE;
		read_buffer_commit += VA_SIZE;
	}
	return TRUE;
}


ItalkConnection::ItalkConnection(void)
{
	int size = VA_SIZE * 256;
	try {
		read_pointer = read_buffer_commit = read_buffer = new char[size];
	} CATCH_BAD_ALLOC;
	read_buffer_reserve = read_buffer + size;
	check_read_buffer();
	ctx = NULL; ssl = NULL; sslMode = false;
	needAnimationStop = false;
}


ItalkConnection::~ItalkConnection()
{
	delete [] read_buffer;
}


BOOL ItalkConnection::Write(const wchar_t* str)
{
	if (!IsValid()) return FALSE;
	if (!str) return TRUE;

	int sizeof_euc_str = lgString::uni2euc((BYTE *)NULL, str);
	char *euc_str = (char *)_alloca(sizeof_euc_str + 1);
	lgString::uni2euc(euc_str, str);
#if 0	// JISM̎gp~
	if (witalk2_ptr->sendJIS){
		euc_str[sizeof_euc_str] = '\0'; /* for safety */
		int length_jis = Quickhack_eucjp_to_jis( NULL, euc_str );
		char *jis_str = (char *)_alloca(sizeof(char) * length_jis);
		(void)Quickhack_eucjp_to_jis( jis_str, euc_str );
		lgSocket::Write(jis_str, strlen(jis_str));
	} else {
#endif
		lgSocket::Write(euc_str, sizeof_euc_str);
#if 0
	}
#endif

	return TRUE;
}

BOOL ItalkConnection::Write(const char* str)
{
	if (!IsValid()) return FALSE;
	if (!str || !(*str)) return TRUE;

	int		sizeof_euc_str = lgString::sjis2euc(NULL, str, FALSE);
	char*	euc_str = (char*)_alloca(sizeof_euc_str + 1);
	lgString::sjis2euc(euc_str, str, FALSE);
	lgSocket::Write(euc_str, sizeof_euc_str);

	return TRUE;
}


void ItalkConnection::Connected(void)
{
	read_pointer = read_buffer;
	*read_pointer = '\0';
	if (icn) icn->Connected(this);
}


void ItalkConnection::FailedToConnect(int wsaerrno)
{
	if (icn) icn->FailedToConnect(this, wsaerrno);
}


void ItalkConnection::Disconnected(void)
{
	if (icn) icn->Disconnected(this);
}


void ItalkConnection::ConnectionClosed(void)
{
	if (icn) icn->ConnectionClosed(this);
}


void ItalkConnection::Error(int wsaerrno)
{
	if (icn) icn->Error(this, wsaerrno);
}


BOOL ItalkConnection::DataArrived(void)
{
	check_read_buffer();
	int len = read_buffer_commit - read_pointer - 1;
	len = lgSocket::Read(read_pointer, len);
	if (0 < len) {
		/* Waffle 0.1.11: 䕶𖕎E
		inid  null ĂĂȂȂȂ͂B */
		for (int i = 0; i < len; i++){
			unsigned char c = read_pointer[i];
			if (c < 0x20 && c != 0x1B && c != 0x0A && c != 0x0D) read_pointer[i] = 0x20;
		}
		read_pointer += len;
		*read_pointer = '\0';
		while ((len = get_line_length())) {
			int length_euc = Quickhack_jis_to_eucjp( NULL, read_buffer );
			//char* eucbuf = (char*)_alloca(sizeof(char) * length_euc);
			char* eucbuf = new char[length_euc];
			(void)Quickhack_jis_to_eucjp( eucbuf, read_buffer );

			int length_buf = lgString::euc2uni(NULL, eucbuf);
			//wchar_t *buf = (wchar_t *)_alloca(sizeof(wchar_t) * (length_buf + 3));
			wchar_t *buf = new wchar_t[length_buf + 3];
			lgString::euc2uni(buf, eucbuf);

			memmove(read_buffer, read_buffer + len, read_pointer - read_buffer - len + 1);
			buf[length_buf    ] = L'\r';
			buf[length_buf + 1] = L'\n';
			buf[length_buf + 2] = L'\0';
			read_pointer -= len;
			if (icn) icn->DataArrived(this, buf);

			delete [] eucbuf;
			delete [] buf;
		}
	}
	return TRUE;
}


// s̃oCg (\0 ܂) Ԃ
int ItalkConnection::get_line_length(void)
{
	char *eol = strchr(read_buffer, '\r');

	if (eol) {
		if (eol[1] == '\n') {
			eol[0] = '\0';
			return eol - read_buffer + 2;
		}
	} else {
		return 0;
	}

	eol = strchr(read_buffer, '\n');
	if (eol) {
		eol[0] = '\0';
		return eol - read_buffer + 1;
	}

	if (read_buffer_reserve - 1 == read_pointer) // obt@܂񂽂
		return read_buffer_reserve - read_buffer;
	else
		return 0;
}
