// Italk Protocol 1.0 and biff-port support modifications.
// Copyright (C) 1998, SAKAMOTO Takahiro <takas@is.s.u-tokyo.ac.jp>

/*
 * WItalk2 - italk client for Win32
 * Copyright (C) 1996-1997, TAGA Nayuta <nayuta@is.s.u-tokyo.ac.jp>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

///////////////////////////////////////////////////////////////////////////////
// ItalkInfo.h


#pragma	once


#include	"iWItalk2.h"
#include	"sys/types.h"
#include	"lgList.h"
#include	<string>
#include	<list>
using namespace std;

class ItalkInfo {

public:

	class Server {
	public:
		wstring		host;
		u_short		port;
		int			users;
		time_t		boottime;
		time_t		currenttime;
		int			uptime;
		wstring		logcode;
		wstring		announce;
		int			subrooms;			// Waffle 0.2.0: Tu[̐ ( 0 ŃTu[)
		bool		is_inid;

		Server(void) { Init(); }
		~Server() { Init(); }
		void Init(void) {
			host = L"";
			port = 12345;
			users = 0;
			subrooms = 0;					// Tu[vgR̓ftHgŎgpȂ
			boottime = 0;
			currenttime = 0;
			uptime = 0;
			logcode = L"";
			announce = L"";
			is_inid = false;
		}
		Server& operator=(const Server &other) {
			if (this != &other) {
				host = other.host;
				port = other.port;
				users = other.users;
				boottime = other.boottime;
				currenttime = other.currenttime;
				uptime = other.uptime;
				logcode = other.logcode;
				announce = other.announce;
			}
			return *this;
		}

		u_short& Port(void) { return port; }
		int& Users(void) { return users; }
		// Waffle 0.2.0: subroom 
		int& SubRoom(void) { return subrooms; }
		time_t& BootTime(void) { return boottime; }
		time_t& CurrentTime(void) { return currenttime; }
		int& UpTime(void) { return uptime; }

		const wchar_t *Host(void) const { 
			if( host.empty() ) return NULL;
			return host.c_str(); 
		}
		const wchar_t *LogCode(void) const {
			if( logcode.empty() ) return NULL;
			return logcode.c_str(); 
		}
		const wchar_t *Announce(void) const { 
			if( announce.empty() ) return NULL;
			return announce.c_str(); 
		}

		const wchar_t *Host(const wchar_t *host) { 
			this->host = host;
			return this->host.c_str(); 
		}
		const wchar_t *LogCode(const wchar_t *logcode) { 
			this->logcode = logcode;
			return this->logcode.c_str();
		}
		const wchar_t *Announce(const wchar_t *announce) {
			this->announce = announce;
			return this->announce.c_str();
		}
	};

	// l̃f[^
	class User
	{
	public:

		class is_userno_ge : public std::unary_function<User, bool> 
		{

		public:
			int comp;
			is_userno_ge( int n ){
				comp = n;
			}
			bool operator( ) ( User& val ) {
				if( val.UserNo() >= comp ) return true;
				return false;
			}
		};

	int		userno;
	int		uptime;
	int		idle;
	time_t	logintime;
	wstring handle;
	wstring host;
	wstring status;
	wstring downcode;
	wstring clientType;	/* Waffle 0.2.0: NCAg` (type=subroom ) */
	User( void ) : userno(-1), uptime(0), idle(0) { Init(); }
	User(int _userno) : uptime(0), idle(0) { 
		Init();
		userno = _userno;
	}
	User(const User &other) : userno(other.userno), uptime(other.uptime), idle(other.idle), handle(other.handle), host(other.host),
							  status(other.status), downcode(other.downcode) {
		}
	~User() { Init(); }
	User& operator=(const User &other) {
		if (this != &other) {
			userno = other.userno;
			uptime = other.uptime;
			idle = other.idle;
			handle = other.handle;
			host = other.host;
			status = other.status;
			downcode = other.downcode;
		}
		return *this;
	}
	void Init(void) {
		int		userno = -1;
		int		uptime = 0;
		int		idle = 0;
		time_t	logintime = -1;
		handle.clear();
		host.clear();
		status.clear();
		downcode.clear();
	}
	int& UserNo(void) { return userno; }
	int& UpTime(void) { return uptime; }
	int& Idle(void) { return idle; }
	time_t& Logintime(void) { return logintime; }
	const wchar_t* Handle(void) const {
		if( handle.empty() ) return NULL;
		return handle.c_str(); 
	}
	const wchar_t* Host(void) const { 
		if( host.empty() ) return NULL;
		return host.c_str(); 
	}
	const wchar_t* Status(void) const { 
		if( status.empty() ) return NULL;
		return status.c_str(); 
	}
	const wchar_t* Downcode(void) const { 
		if( downcode.empty() ) return NULL;
		return downcode.c_str(); 
	}
	const wchar_t* ClientType(void) const { 
		if( clientType.empty() ) return NULL;
		return clientType.c_str(); 
	}
		const wchar_t* Handle(const wchar_t *handle) {
		this->handle = handle;
		return this->handle.c_str(); 
	}
	const wchar_t *Host  (const wchar_t *host) {
		this->host = host;
		return this->host.c_str();
	}
	const wchar_t *Status(const wchar_t *status) {
		this->status = status;
		return this->status.c_str();
	}
	const wchar_t* Downcode(const wchar_t *downcode) {
		this->downcode = downcode;
		return this->downcode.c_str();
	}
	const wchar_t* ClientType(const wchar_t *clientType) {
		this->clientType = clientType;
		return this->clientType.c_str();
	}
};

enum InWhich { in_nowhere, in_user, in_you, in_server };

class MiniInfo {

public:

	BOOL	isInInfo;		// <italk> ̒ɂ邩ǂ
    BOOL	isInNewUser;	// <newuser> ̒ɂ邩ǂ
    InWhich	in_which;		// <user> <you> <server> ̂ǂɂ邩
    Server	server;			// ݏ <server> </server>
	//lgList<User> users;	// ݏ <italk> </italk>
	list<User> users;
	User temp_user;			// ݏ <user> </user>
	int my_userno;			// ݏ <you> </you>

	MiniInfo( void ) { isInInfo = isInNewUser = FALSE; }
	MiniInfo& operator=(const MiniInfo& other) {
		if (this != &other) {
			isInInfo = other.isInInfo;
			isInNewUser = other.isInNewUser;
			in_which = other.in_which;
			server = other.server;
			temp_user = other.temp_user;
			my_userno = other.my_userno;
			/*
			while ( users.Length() )
			users.Delete(0);
			for (int i = 0; i < other.users.Length(); i++)
			users.Add(other.users[i]);
			*/
			users = other.users;
		}
		return *this;
	}
};

public:
	Server		server;		// italk server ɂĂ̏
	//lgList<User> users;	// users Xg
	list<User>	users;
	int			my_userno;	//  userno

	MiniInfo infoMain, infoBiff;	// ݏ <italk>
	MiniInfo *info;			// ݏ <italk> ւ̃|C^

	int search_userno( lgList<User> &users, int userno );	// ǉ
	User* search_userno( list<User> &users, int userno );///쐬 Userւ̃|C^Ԃ
	BOOL begin_italk_info_mode(void);			// ItalkInfoMode ɓ˓
	BOOL end_italk_info_mode(void);			// ItalkInfoMode A
	BOOL parse_italk_info_string(const wchar_t *str);	// ItalkInfoMode 
	BOOL parse_newuser_string(const wchar_t *str);	// newuser 
	BOOL parse_one_line(const wchar_t *str);		// sXe[^X

public:
	ItalkInfo(void);
	ItalkInfo& operator=(const ItalkInfo &other);
	int Check(const wchar_t *str, BOOL isMain = TRUE );	// ItalkInfoMode ̕񂩂`FbN
	void Reset(void);                     // Zbg
	int GetNextUserNo(int userno);        // ̃[U[ԍ𓾂
	int GetPrevUserNo(int userno);        // Õ[U[ԍ𓾂
	int GetRightUserNo(int userno);       // [U[ԍ𓾂
	const wchar_t *GetHandle(int userno); // [U[ԍnh𓾂
	void Raise(int userno);               // 擪ɎĂ
	User* GetUser(int userno);            // [U[ԍ烆[U[𓾂
	User* GetUserFromIndex(int index);    // CfbNXԍ烆[U[𓾂
	BOOL IsInItalkInfoMode(void);         // ItalkInfoMode ǂ
	Server* GetServer(void);              // italk server ̏𓾂
	int WhoAmI(void);                     // ̃[U[ԍԂ
	wchar_t* GetUsersStringForDebug(int i, wchar_t *buf, int sizeof_buf); // fobOp
};

#undef USERNO_SHIFT
