/*
 * PrefReg - Preference Registry manager
 * Copyright (c) 2002 by Project Waffle / TSUJIKAWA Tohru <tsujikawa@tsg.ne.jp>
 *  
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

///////////////////////////////////////////////////////////////////////////////
// PrefReg.cpp


#include	<winsock2.h>
#include	<algorithm>
using namespace std;

#include	<shlwapi.h>

#include	"PrefReg.h"

//
// constructor
//
PrefReg::PrefReg(const char* _keyname, bool doCreate, HKEY _keyroot)
{
	keyname.assign(_keyname); keyroot = _keyroot;
	if (exist() == PREFREG_NOSUCHKEYVALUE) create();
}

//
// ̃L[݂邩ǂԂ
//
PrefRegError PrefReg::exist()
{
	HKEY	thiskey;
	if (RegOpenKeyEx(keyroot, keyname.c_str(), 0, KEY_READ, &thiskey) == ERROR_SUCCESS) {
		RegCloseKey(thiskey);
		return PREFREG_SUCCESS;
	} else return PREFREG_NOSUCHKEYVALUE;
}

//
// ̃L[݂Ȃ΍쐬
//
PrefRegError PrefReg::create()
{
	HKEY	thiskey;
	if (RegCreateKeyEx(keyroot, keyname.c_str(), 0, 0, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &thiskey, NULL) == ERROR_SUCCESS) {
		RegCloseKey(thiskey);
		return PREFREG_SUCCESS;
	} else return PREFREG_WRITEFAIL;
}

//
// keyJ
//
HKEY PrefReg::readopen()
{
	HKEY	thiskey;
	if (RegOpenKeyEx(keyroot, keyname.c_str(), 0, KEY_READ, &thiskey) == ERROR_SUCCESS) {
		return thiskey;
	} else return NULL;
}

HKEY PrefReg::writeopen()
{
	HKEY	thiskey;
	if (RegOpenKeyEx(keyroot, keyname.c_str(), 0, KEY_WRITE, &thiskey) == ERROR_SUCCESS) {
		return thiskey;
	} else return NULL;
}

//
// keyclose
//
void PrefReg::close(HKEY key)
{
	RegCloseKey(key);
}

//
// DWORD lǂ
//
DWORD PrefReg::readDword(const char* valname, DWORD defvalue)
{
	HKEY	key;
	if (key = readopen()) {
		DWORD	r;
		DWORD	rlen = sizeof(r);
		if (RegQueryValueEx(key, valname, NULL, NULL, (LPBYTE)&r, &rlen) == ERROR_SUCCESS) return r;
	}
	return defvalue;
}

//
// DWORD l
//
PrefRegError PrefReg::writeDword(const char* valname, DWORD value)
{
	HKEY	key;
	if (key = writeopen()) {
		if (RegSetValueEx(key, valname, NULL, REG_DWORD, (const BYTE*)&value, sizeof(value)) == ERROR_SUCCESS) return PREFREG_SUCCESS;
		return PREFREG_WRITEFAIL;
	}
	return PREFREG_NOSUCHKEYVALUE;
}

//
// ǂ
//
string PrefReg::readString(const char* valname, const char* defvalue)
{
	HKEY	key;
	if (key = readopen()) {
		char	buf[1024];
		DWORD	buflen = sizeof(buf);
		if (RegQueryValueEx(key, valname, NULL, NULL, (LPBYTE)buf, &buflen) == ERROR_SUCCESS) return string(buf);
	}
	return string(defvalue);
}

//
// 
//
PrefRegError PrefReg::writeString(const char* valname, const char* value)
{
	HKEY	key;
	if (key = writeopen()) {
		if (RegSetValueEx(key, valname, NULL, REG_SZ, (const BYTE*)value, strlen(value)+1) == ERROR_SUCCESS) return PREFREG_SUCCESS;
		return PREFREG_WRITEFAIL;
	}
	return PREFREG_NOSUCHKEYVALUE;
}

//
// subkey݊mF
//
PrefRegError PrefReg::existSubkey(const char* subkeyname)
{
	PrefReg		prefreg(keyname+string(subkeyname), false, keyroot);
	return prefreg.exist();
}

//
// value݊mF
//
PrefRegError PrefReg::existValue(const char* valname)
{
	vector<string>	vs = enumValue();
	if (find(vs.begin(), vs.end(), string(valname)) != vs.end()) return PREFREG_SUCCESS;
	return PREFREG_NOSUCHKEYVALUE;
}

//
// subkey, value폜
//
PrefRegError PrefReg::deleteAll()
{
	if (exist() == PREFREG_NOSUCHKEYVALUE) return PREFREG_NOSUCHKEYVALUE;
	if (SHDeleteKey(keyroot, keyname.c_str()) == ERROR_SUCCESS) return PREFREG_SUCCESS;
	return PREFREG_WRITEFAIL;
}

//
// subkey폜
//
PrefRegError PrefReg::deleteSubkey(const char* subkeyname)
{
	if (exist() == PREFREG_NOSUCHKEYVALUE) return PREFREG_NOSUCHKEYVALUE;
	if (SHDeleteKey(keyroot, (keyname+string(subkeyname)).c_str()) == ERROR_SUCCESS) PREFREG_SUCCESS;
	return PREFREG_WRITEFAIL;
}

//
// value폜
//
PrefRegError PrefReg::deleteValue(const char* valname)
{
	HKEY	key;
	if (key = writeopen()) {
		if (RegDeleteValue(key, valname) == ERROR_SUCCESS) return PREFREG_SUCCESS;
		return PREFREG_WRITEFAIL;
	}
	return PREFREG_NOSUCHKEYVALUE;
}

//
// subkey
//
vector<string> PrefReg::enumSubkey()
{
	vector<string>	r;
	string			a;
	char			buf[1024];
	DWORD			buflen = sizeof(buf);
	HKEY			key = readopen();
	FILETIME		dmy;
	if (key) for (int i = 0; ; i++) {
		if (RegEnumKeyEx(key, i, buf, &buflen, NULL, NULL, NULL, &dmy) != ERROR_SUCCESS) break;
		a.assign(buf);
		r.push_back(a);
	}
	return r;
}

//
// value
//
vector<string> PrefReg::enumValue()
{
	vector<string>	r;
	string			a;
	char			buf[1024];
	DWORD			buflen = sizeof(buf);
	HKEY			key = readopen();
	if (key) for (int i = 0; ; i++) {
		if (RegEnumValue(key, i, buf, &buflen, NULL, NULL, NULL, NULL) != ERROR_SUCCESS) break;
		a.assign(buf);
		r.push_back(a);
	}
	return r;
}
