/*
 * WItalk2 - italk client for Win32
 * Copyright (C) 1996-1997, TAGA Nayuta <nayuta@is.s.u-tokyo.ac.jp>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

///////////////////////////////////////////////////////////////////////////////
// Sound.h


#pragma	once

#include	<mmsystem.h>


class Sound
{
  int sound;
  char *filename;

public:
  Sound(void)
    : sound(-3),
      filename(NULL)
  {
  }

  ~Sound()
  {
    delete_string(filename);
  }

  Sound &operator=(const Sound &other)
  {
    sound = other.sound;
    replace_string(filename, other.filename);
    return *this;
  }
  
  char *Encode(char *buf, int sizeof_buf)
  {
    if (sound == -2 && filename)
    {
      if (sizeof_buf <= (int)strlen(filename))
	return NULL;
      strcpy(buf, filename);
    }
    else if (sound == -2 || sound == -3)
    {
      if (sizeof_buf < 1)
	return NULL;
      *buf = '\0';
    }
    else
    {
      int r;
      char n;
      switch (sound)
      {
	default:
	case -1:                 n = 'B'; r = rStringSoundBeep;        break;
	case MB_ICONHAND:        n = 'H'; r = rStringSoundHand;        break;
	case MB_OK:              n = 'O'; r = rStringSoundOk;          break;
	case MB_ICONEXCLAMATION: n = 'E'; r = rStringSoundExclamation; break;
	case MB_ICONQUESTION:    n = 'Q'; r = rStringSoundQuestion;    break;
	case MB_ICONASTERISK:    n = 'A'; r = rStringSoundAsterisk;    break;
      }
      char *name = lgWindows::LoadString(r);
      if (sizeof_buf <= (int)strlen(name) + 5)
	return NULL;
      sprintf(buf, "<%s(%c)>", name, n);
    }
    return buf;
  }
  
  BOOL Decode(const char *buf)
  {
    if (buf == NULL || buf[0] == '\0')
      sound = -3;
    else if (buf[0] == '<')
    {
      int len = strlen(buf);
      if (len < 5)
	return FALSE;
      switch (buf[len - 3])
      {
	default:
	case 'B': sound = -1;                 break; // Beep 
	case 'H': sound = MB_ICONHAND;        break; // VXe G[
	case 'O': sound = MB_OK;              break; // ʂ̌x
	case 'E': sound = MB_ICONEXCLAMATION; break; // bZ[W - x
	case 'Q': sound = MB_ICONQUESTION;    break; // bZ[W - ₢킹
	case 'A': sound = MB_ICONASTERISK;    break; // bZ[W - 
      }
    }
    else
    {
      sound = -2;
      replace_string(filename, buf);
    }
    return TRUE;
  }
  
  void Play(void)
  {
    if (sound == -2)
    {
      if (filename)
	PlaySound(filename, NULL, SND_ASYNC | SND_FILENAME | SND_NOWAIT);
    }
    else if (sound != -3)
      MessageBeep(sound);
  }
  
  void Set(int sound, const char *filename = NULL)
  {
    this->sound = sound;
    if (filename && *filename)
      replace_string(this->filename, filename);
  }

  int GetSound(void) { return sound; }
  const char *GetFilename(void) { return filename; }
};
