/*
 * TimerQueue - Timer callback handler
 * Copyright (c) 2012 by Project Waffle / TSUJIKAWA Tohru <tsujikawa@tsg.ne.jp>
 *  
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

///////////////////////////////////////////////////////////////////////////////
// TimerQueue.h

#pragma	once

#ifndef _TIMER_QUEUE_H_
#define	_TIMER_QUEUE_H_

#include	<Windows.h>

#include	<time.h>

#include	<memory>
#include	<map>
#include	<vector>

#ifndef CREATE_MUTEX
#define		CREATE_MUTEX()				CreateMutex(NULL, FALSE, NULL)
#define		DESTROY_MUTEX(m)			CloseHandle(m)
#define		MUTEX_LOCK(m)				do { WaitForSingleObject(m, INFINITE); } while (false)
#define		MUTEX_UNLOCK(m)				do { ReleaseMutex(m); } while (false)
#endif // !CREATE_MUTEX

#define		WM_TIMERQUEUE				(WM_APP + 130)


class TimerQueueTimer;
typedef		std::shared_ptr<TimerQueueTimer>		pTimerQueueTimer;

class TimerQueue;
typedef		std::shared_ptr<TimerQueue>				pTimerQueue;

extern TimerQueue*	wi_timer_q;


class TimerQueueTimer {

public:

	HANDLE	timer;
	TimerQueue*		parent_timer_q;

	DWORD	id;										// ^C}CxgID
	DWORD	cat1;									// ^C}Cxg JeS1x
	DWORD	cat2;									// ^C}Cxg JeS2x

	unsigned long long	triggered_time;				// ^C}Cxg (bP)
	unsigned long long	expected_firing_time;		// ^C}Cxg\莞 (bP)

	unsigned long long	duration_msec;				// ҋ@ (~b)

	void*	param;									// EBhEvV[Wɑp[^

	TimerQueueTimer ();
	~TimerQueueTimer () {}

	static pTimerQueueTimer new_inst () {
		return pTimerQueueTimer(new TimerQueueTimer);
	}

};
typedef		std::shared_ptr<TimerQueueTimer>		pTimerQueueTimer;


class TimerQueue {

	HANDLE	mutex;

	HANDLE	timer_q;
	HWND	hwnd_message_sent_to;

	DWORD	next_id;

	std::map<DWORD, pTimerQueueTimer>	timers;

	static void WINAPI WaitOrTimerCallback (PVOID lpParameter, BOOL TimerOrWaitFired);

	DWORD get_next_id ();

public:

	TimerQueue (HWND _hwnd_message_sent_to);
	~TimerQueue ();

	DWORD start (DWORD _cat1, DWORD _cat2, unsigned long long duration_msec, void* param);
	bool restart (DWORD _cat1, DWORD _cat2, unsigned long long new_duration_msec);
	bool cancel (DWORD _id);
	bool cancel (DWORD _cat1, DWORD _cat2);

};


#endif // !_TIMER_QUEUE_H_
