/*
 * LIBGANA - library for GANA
 * Copyright (C) 1996-1997, TAGA Nayuta <nayuta@is.s.u-tokyo.ac.jp>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

///////////////////////////////////////////////////////////////////////////////
// lgKeyBindingsMap.cxx


#include "lgKeyBindingsMap.h"


// 𓾂
HWND lgKeyBindingsMap::get_hwnd_dest(int flag)
{
  if ((flag & Dest) == DestParent)
    return GetParent(Hwnd());
  else if ((flag & Dest) == DestOwner)
    return HwndOwner();
  else
    return Hwnd();
}


// RXgN^
lgKeyBindingsMap::lgKeyBindingsMap(HWND hwnd)
  : lgWindow(hwnd),
    tmp_map(NULL)
{
  Clear();
}

lgKeyBindingsMap::lgKeyBindingsMap(const lgKeyBindingsMap &other)
  : lgWindow(other.Hwnd()),
    tmp_map(other.tmp_map)
{
  Copy(other);
}
  

// map NA
void lgKeyBindingsMap::Clear(void)
{
  int i;
  for (i = 0; i < ArrayNum(vkey_map); i++)
    for (int j = 0; j < ArrayNum(vkey_map[0]); j++)
      vkey_map[i][j].flag = 0;
    
  for (i = 0; i < ArrayNum(char_map); i++)
    for (int j = 0; j < ArrayNum(char_map[0]); j++)
      char_map[i][j].flag = 0;
}


// vkey_map ւ̊蓖 (w肳ꂽ shift L[݂̂ꂽ)
void lgKeyBindingsMap::SetVKey(int vk_code, int shift,
					 int flag, int message,
					 WPARAM wParam, LPARAM lParam)
{
  Message &m = vkey_map[vk_code & 0xff][shift & 7];
  m.flag    = flag;
  m.message = message;
  m.wParam  = wParam;
  m.lParam  = lParam;
}


// vkey_map ւ̊蓖 (w肳ꂽ shift L[Ă΂悢)
void lgKeyBindingsMap::SetVKeyOr(int vk_code, int shift,
					   int flag, int message,
					   WPARAM wParam, LPARAM lParam)
{
  for (int i = 0; i < 8; i++)
    SetVKey(vk_code, shift | i, flag, message, wParam, lParam);
}


// vk_codes ɗ񋓂Ă鉼zL[R[h̊蓖
void lgKeyBindingsMap::SetVKeyOver(BYTE *vk_codes, int n,
					     int shift, int flag, int message,
					     WPARAM wParam, LPARAM lParam)
{
  for (int i = 0; i < n; i++)
    SetVKey(vk_codes[i], shift, flag, message, wParam, lParam);
}
  

// char_map ւ̊蓖
void lgKeyBindingsMap::SetChar(int char_code, int shift,
					 int flag, int message,
					 WPARAM wParam, LPARAM lParam)
{
  char_code &= 0xff;
    
  if (shift & Shift)
    if ('a' <= char_code && char_code <= 'z')
      char_code += 'A' - 'a';
    else if ('A' <= char_code && char_code <= 'Z')
      char_code += 'a' - 'A';
    
  if (shift & Control)
    if (char_code == '?')
      char_code = '\x7f';
    else
      char_code %= 0x20;
    
  Message &m = char_map[char_code][(shift & Alt) ? 1 : 0];
  m.flag    = flag;
  m.message = message;
  m.wParam  = wParam;
  m.lParam  = lParam;
}


// char_map ւ̘A蓖
void lgKeyBindingsMap::SetCharOver(int char_code_from,
					     int char_code_to,
					     int shift, int flag, int message,
					     WPARAM wParam, LPARAM lParam)
{
  for (int i = char_code_from; i <= char_code_to; i++)
    SetChar(i, shift, flag, message, wParam, lParam);
}


// char_codes ɗ񋓂ĂR[h̊蓖
void lgKeyBindingsMap::SetCharOver(char *char_codes, int n,
					     int shift, int flag, int message,
					     WPARAM wParam, LPARAM lParam)
{
  for (int i = 0; i < n; i++)
    SetChar(char_codes[i], shift, flag, message, wParam, lParam);
}


//  lgKeyBindingsMap ̃Rs[
lgKeyBindingsMap &lgKeyBindingsMap::Copy
(const lgKeyBindingsMap &other)
{
  int i;
  for (i = 0; i < ArrayNum(vkey_map); i++)
    for (int j = 0; j < ArrayNum(vkey_map[0]); j++)
      vkey_map[i][j] = other.vkey_map[i][j];
    
  for (i = 0; i < ArrayNum(char_map); i++)
    for (int j = 0; j < ArrayNum(char_map[0]); j++)
      char_map[i][j] = other.char_map[i][j];

  return *this;
}


// bZ[WnhO

//  TRUE
BOOL lgKeyBindingsMap::KeyDown(int vk_code)
{
  if (tmp_map)
  {
    lgKeyBindingsMap *tm = tmp_map;
    tmp_map = NULL;
    if (tm->KeyDown(vk_code))
      return TRUE;
    tmp_map = tm;
    return FALSE;
  }
    
  vk_code &= 0xff;
    
  int shift = 0;
  shift |= (GetKeyState(VK_SHIFT  ) < 0) ? Shift   : 0;
  shift |= (GetKeyState(VK_CONTROL) < 0) ? Control : 0;
  shift |= (GetKeyState(VK_MENU   ) < 0) ? Alt     : 0;
  Message &m = vkey_map[vk_code][shift];
    
  if ((m.flag & Dest) == DestNone)
    return FALSE;
    
  WPARAM wParam = m.wParam;
  if ((m.flag & wParamCode) && (m.flag & wParamShift))
    wParam = vk_code | (shift << 8);
  else if (m.flag & wParamCode)
    wParam = vk_code;
  else if (m.flag & wParamShift)
    wParam = shift;
      
  LPARAM lParam = m.lParam;
  if ((m.flag & lParamCode) && (m.flag & lParamShift))
    lParam = vk_code | (shift << 8);
  else if (m.flag & lParamCode)
    lParam = vk_code;
  else if (m.flag & lParamShift)
    lParam = shift;
    
  SendMessage(get_hwnd_dest(m.flag), m.message, wParam, lParam);

  return TRUE;
}

BOOL lgKeyBindingsMap::SysKeyDown(int vk_code)
{
  return KeyDown(vk_code);
}
  

//  TRUE
BOOL lgKeyBindingsMap::Char(char char_code, BOOL is_alt_pressed)
{
  if (tmp_map)
  {
    lgKeyBindingsMap *tm = tmp_map;
    tmp_map = NULL;
    return tm->Char(char_code, is_alt_pressed);
  }
    
  Message &m = char_map[(BYTE)char_code][is_alt_pressed ? 1 : 0];
    
  if ((m.flag & Dest) == DestNone)
    return FALSE;
    
  WPARAM wParam = m.wParam;
  if (m.flag & wParamCode)
    wParam = (BYTE)char_code;
  else if (m.flag & wParamShift)
    wParam = 0;
    
  LPARAM lParam = m.lParam;
  if (m.flag & lParamCode)
    lParam = (BYTE)char_code;
  else if (m.flag & lParamShift)
    lParam = 0;

  SendMessage(get_hwnd_dest(m.flag), m.message, wParam, lParam);
    
  return TRUE;
}

BOOL lgKeyBindingsMap::SysChar(int vk_code)
{
  return Char(vk_code, TRUE);
}


// ꎞIɃL[}bv؂ւ
lgKeyBindingsMap *lgKeyBindingsMap::SetTmpMap
(lgKeyBindingsMap *tmp_map)
{
  lgKeyBindingsMap *old_tmp_map = this->tmp_map;
  this->tmp_map = tmp_map;
  return old_tmp_map;
}
