/*
 * LIBGANA - library for GANA
 * Copyright (C) 1996-1997, TAGA Nayuta <nayuta@is.s.u-tokyo.ac.jp>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

///////////////////////////////////////////////////////////////////////////////
// lgRegistry


#pragma	once


#include "lgWindows.h"


class lgRegistry
{
  HKEY root;  // WXg[g
  char *path; // WXg[g̃pX
  
public:
  lgRegistry(void)
    : root(NULL),
      path(NULL) { }
  
  lgRegistry(HKEY _root, const char *_path)
    : path(NULL)
  {
    Set(_root, _path);
  }
  
  ~lgRegistry()
  {
    delete_string(path);
  }
  
  // WXg[gƃpX̍Đݒ
  void Set(HKEY _root, const char *_path)
  {
    root = _root;
    delete_string(path);
    path = new_string(_path);
  }
  
  // 폜
  BOOL Delete(const char * const name)
  {
    return Delete(root,  path, name);
  }

  // Waffle 0.1.5
  BOOL CopyDwordINI(const char* const name);
  BOOL CopyStringINI(const char* const name);

  // Waffle 0.2.0f:
  static BOOL CopyStringReg(HKEY root, const char* const path, const char* const name);
 
  // DWORD ǂ
  BOOL Read(const char * const name, int &buf, int default_value = 0)
  {
    return Read(root, path, name, buf, default_value);
  }

  // DWORD 
  BOOL Write(const char * const name, int value)
  {
    return Write(root, path, name, value);
  }
 
  // ǂ
  BOOL Read(const char * const name, char *buf, const DWORD sizeof_buf,
	    const char *default_str = NULL)
  {
    return Read(root, path, name, buf, sizeof_buf, default_str);
  }

  // 
  BOOL Write(const char * const name, const char * const value)
  {
    return Write(root, path, name, value);
  }

  // Chǂ
  BOOL Read(const char * const name, wchar_t *buf, const DWORD sizeof_buf,
	    const wchar_t *default_str = NULL)
  {
    return Read(root, path, name, buf, sizeof_buf, default_str);
  }

  // Ch
  BOOL Write(const char * const name, const wchar_t * const value)
  {
    return Write(root, path, name, value);
  }
  
  // oCiǂ
  BOOL Read(const char * const name, BYTE *buf, DWORD &sizeof_buf,
	    const BYTE *default_binary = NULL, DWORD sizeof_default_binary = 0)
  {
    return Read(root, path, name, buf, sizeof_buf, default_binary,
		sizeof_default_binary);
  }

  // oCi
  BOOL Write(const char * const name, const BYTE * const value,
	     DWORD sizeof_value)
  {
    return Write(root, path, name, value, sizeof_value);
  }

  // LOGFONT ǂ
  BOOL Read(const char * const name, LOGFONT &lf,
	    const char * const default_string_logfont = NULL)
  {
    return Read(root, path, name, lf, default_string_logfont);
  }

  // LOGFONT 
  BOOL Write(const char * const name, const LOGFONT &lf)
  {
    return Write(root, path, name, lf);
  }
  
  // RECT ǂ
  BOOL Read(const char * const name, RECT &rc, const RECT &default_rc)
  {
    return Read(root, path, name, rc, default_rc);
  }

  // RECT 
  BOOL Write(const char * const name, const RECT &rc)
  {
    return Write(root, path, name, rc);
  }
  
public:
  
#define lgRegistry_path \
  const HKEY root, const char * const path, const char * const name
  
  // 폜
  static BOOL Delete(lgRegistry_path);
  
  // DWORD
  static BOOL Read(lgRegistry_path, int &buf, int default_value = 0);
  static BOOL Write(lgRegistry_path, int value);

  // 
  static BOOL Read(lgRegistry_path,
			     char *buf, const DWORD sizeof_buf,
			     const char *default_str = NULL);
  static BOOL Write(lgRegistry_path, const char * const value);
  
  // Ch
  static BOOL Read(lgRegistry_path,
			     wchar_t *buf, const DWORD sizeof_buf,
			     const wchar_t *default_str = NULL);
  static BOOL Write(lgRegistry_path, const wchar_t * const value);
  
  // oCi
  static BOOL Read(lgRegistry_path, BYTE *buf, DWORD &sizeof_buf,
			     const BYTE *default_binary = NULL,
			     DWORD sizeof_default_binary = 0);
  static BOOL Write(lgRegistry_path,
			      const BYTE * const value, DWORD sizeof_value);
  
  // LOGFONT
  static BOOL Read(lgRegistry_path, LOGFONT &lf,
			     const char * const default_string_logfont = NULL);
  static BOOL Write(lgRegistry_path, const LOGFONT &lf);
  
  // RECT
  static BOOL Read(lgRegistry_path,
			     RECT &rc, const RECT &default_rc);
  static BOOL Write(lgRegistry_path, const RECT &rc);
#undef lgRegistry_path
  
  // ֘AtꂽWXgڂ𓾂
  static BOOL GetAssociated(const char *type, const char *path,
				      char *buf, int sizeof_buf);
  
  // ֘AtR}hC𓾂
  static BOOL GetAssociatedCommandline
  (const char *type, char *commandline,
   int sizeof_commandline, const char *action = "open");
  
  // ֘AtR}hC̉
  static char *InterpretAssociatedCommandline
  (const char *filename, const char *commandline, char *buf, int sizeof_buf);
  
  // ֘AtACR𓾂
  static HICON GetAssociatedIcon
  (const char *type, const char *filename = NULL);
};
