/*
 * LIBGANA - library for GANA
 * Copyright (C) 1996-1997, TAGA Nayuta <nayuta@is.s.u-tokyo.ac.jp>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

///////////////////////////////////////////////////////////////////////////////
// lgcStatubar.cxx


#include "lgWindows.h"


lgcStatusbar::lgcStatusbar(HWND hwnd)
  : lgWindow(hwnd),
    default_text(NULL),
    now_text(new_string("")),
    fg((COLORREF)~0),
    bg((COLORREF)~0),
    is_tmp(FALSE)
{
}


lgcStatusbar::~lgcStatusbar()
{
  SetColor();
  delete [] default_text;
  delete [] now_text;
}


const char *lgcStatusbar::SetDefaultText(const char *text)
{
  const char *old = default_text;
  replace_string(default_text, text);
  SetText(NULL);
  return old;
}


void lgcStatusbar::SetText(const char *text)
{
  if (!text)
    if (default_text)
      text = default_text;
    else
      text = "";
  replace_string(now_text, text);
  if (is_tmp)
    SetColor();
  else if (fg == ~0)
    SendMessage(SB_SETTEXT, 0, (LPARAM)now_text);
  else
    SendMessage(SB_SETTEXT, SBT_OWNERDRAW);
}


void lgcStatusbar::SetColor(void)
{
  fg = bg = (COLORREF)~0;
  SendMessage(SB_SETTEXT, 0, (LPARAM)now_text);
}


void lgcStatusbar::SetColor(COLORREF _fg, COLORREF _bg, BOOL _is_tmp)
{
  fg = _fg;
  bg = _bg;
  is_tmp = _is_tmp;
  SendMessage(SB_SETTEXT, SBT_OWNERDRAW);
}


// I[i[h[
int lgcStatusbar::wmDrawItem(int id, DRAWITEMSTRUCT *dis)
{
  if (dis->hwndItem == Hwnd())
  {
    RECT rc = dis->rcItem;

    HBRUSH hbrush = CreateSolidBrush(bg);
    FillRect(dis->hDC, &rc, hbrush);
    DeleteObject(hbrush);

    fg = SetTextColor(dis->hDC, fg);
    bg = SetBkColor(dis->hDC, bg);
    DrawText(dis->hDC, now_text, -1, &rc,
	     DT_LEFT | DT_SINGLELINE | DT_NOPREFIX | DT_VCENTER);
    fg = SetTextColor(dis->hDC, fg);
    bg = SetBkColor(dis->hDC, bg);
  }
  return TRUE;
}
